<?php

namespace Drupal\commerce_tpay\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_tpay\Event\TpayPaymentEvent;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\commerce_price\Price;
use Drupal\commerce_order\Entity\Order;
use Drupal\Core\Url;

/**
 * Provides the Off-site Redirect payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "tpay_redirect",
 *   label = "Tpay Redirect",
 *   display_label = "Tpay Redirect",
 *    forms = {
 *     "offsite-payment" = "Drupal\commerce_tpay\PluginForm\TpayRedirect\TpayPaymentForm",
 *   },
 * )
 */
class TpayRedirect extends OffsitePaymentGatewayBase {
  
  /**
   * The logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;
  
  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;
  
  /**
   * The time.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;
  
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, PaymentTypeManager $payment_type_manager, PaymentMethodTypeManager $payment_method_type_manager, LoggerChannelFactoryInterface $logger_channel_factory, EventDispatcherInterface $event_dispatcher, TimeInterface $time) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time);
    $this->logger = $logger_channel_factory->get('commerce_tpay');
    $this->eventDispatcher = $event_dispatcher;
    $this->time = $time;
  }
  
  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('logger.factory'),
      $container->get('event_dispatcher'),
      $container->get('datetime.time')
    );
  }
  
  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $sid = $this->configuration['sid'];
    $confirmation_code = $this->configuration['confirmation_code'];
    
    $form['sid'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Identyfikator Odbiorcy'),
      '#default_value' => $sid,
      '#description' => $this->t('Liczbowy identyfikator Odbiorcy Płatności nadany podczas rejestracji'),
      '#required' => TRUE,
    ];
    
    $form['confirmation_code'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Kod potwierdzający'),
      '#default_value' => $confirmation_code,
      '#description' => $this->t('Kod potwierdzający ustawiany jest w Panelu sprzedawcy w zakładce Ustawienia > Powiadomienia, sekcja Bezpieczeństwo'),
      '#required' => TRUE,
    ];
    
    return $form;
  }
  
  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['sid'] = $values['sid'];
      $this->configuration['confirmation_code'] = $values['confirmation_code'];
    }
  }
  
  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    drupal_set_message($this->t('Twoje zamówienie o numerze: @orderid zakończyło się powodzeniem', ['@orderid' => $order->id()]));
  }
  
  /**
   * {@inheritdoc}
   */
  public function onCancel(OrderInterface $order, Request $request) {
    drupal_set_message($this->t('Płatność @gateway nieudana, można wznowić proces w dowolnym momencie.', [
      '@gateway' => $this->getDisplayLabel(),
    ]), 'error');
  }
  
  /**
   * {@inheritdoc}
   */
  public function onNotify(Request $request) {
    $sid = $this->configuration['sid'];
    $confirmation_code = $this->configuration['confirmation_code'];
    
    $tr_id = $request->get('tr_id');
    $tr_amount = $request->get('tr_amount');
    $tr_crc = $request->get('tr_crc');
    $tr_paid = $request->get('tr_paid');
    $tr_status = $request->get('tr_status');
    $tr_error = $request->get('tr_error');
    $md5sum = $request->get('md5sum');
    
    $crc_pieces = explode("/", $tr_crc);
    $order_id = $crc_pieces[0];
    $currency = $crc_pieces[1];
    
    if ($md5sum == md5($sid . $tr_id . $tr_amount . $tr_crc . $confirmation_code)) {
      if ($tr_status == 'TRUE') {
        $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
        $order = Order::load($order_id);
        $existing_payments = $payment_storage->loadMultipleForOrder($order);
        
        $payment_logged = FALSE;
        foreach ($existing_payments as $payment) {
          if ($payment->getRemoteId() == $tr_id) {
            $payment_logged = TRUE;
            break;
          }
        }
        
        if (!$payment_logged) {
          $payment = $payment_storage->create([
            'state' => 'completed',
            'amount' => new Price($tr_paid, $currency),
            'payment_gateway' => $this->entityId,
            'order_id' => $order_id,
            'remote_id' => $tr_id,
            'remote_state' => $tr_error,
            'completed' => $this->time->getRequestTime(),
          ]);
          
          $payment->save();
          
          $event = new TpayPaymentEvent($payment);
          
          $this->eventDispatcher->dispatch(TpayPaymentEvent::TPAY_PAYMENT_RECEIVED, $event);
        }
      }
    } else {
      $this->logger->alert($this->t('Niepoprawna suma kontrolna'));
    }
    
    $response = new Response('TRUE', Response::HTTP_OK);
    return $response;
  }
  
  /**
   * {@inheritdoc}
   */
  public function getNotifyUrl() {
    return Url::fromRoute('commerce_payment.notify', [
      'commerce_payment_gateway' => $this->entityId,
    ], ['absolute' => TRUE])->toString();
  }
  
  public function getReturnUrl(OrderInterface $order) {
    return Url::fromRoute('commerce_payment.checkout.return', [
      'commerce_order' => $order->id(),
      'step' => 'payment',
    ], ['absolute' => TRUE])->toString();
  }
  
  public function getCancelUrl(OrderInterface $order) {
    return Url::fromRoute('commerce_payment.checkout.cancel', [
      'commerce_order' => $order->id(),
      'step' => 'payment',
    ], ['absolute' => TRUE])->toString();
  }
  
}
